import csv
import DataRow
import codecs
import sys


class LoadCsvData:
    """ A class for loading csv data into a data row. """

    rows = []
    rowCount = 0

    limitLoad = True
    rowLimit = 25

    cleanData = False
    cleanCount = 0

    verbose = False

    def __init__(self, lRows=[], lLimitLoad=False, lRowLimit=-1, lCleanData=False, lVerbose=False):
        self.rows = lRows
        self.limitLoad = lLimitLoad
        self.rowLimit = lRowLimit
        self.cleanData = lCleanData
        self.verbose = lVerbose
    #edef

    def loadData(self, csvFile='', type='csv', version='1.0', reset=False, dataMap={}, appendCols={}):
        print ("")
        print ("")
        print("Loading Data: " + csvFile + " Type: " + type + " Version: " + version + " Reset: " + str(reset))

        if self.verbose:
            print ("Found data mapping:")
            for i in dataMap:
                print ("%i, %s" % (i, dataMap[i]))
            #efl
        #eif

        if self.verbose:
            print ("Found append cols mapping:")
            for i in appendCols:
                print(i)
            #efl
        #eif

        if reset == True:
            print ("Resetting rows:")
            self.resetRows()
        #eif

        if type == 'csv' and version == '1.0' and csvFile != '':
            ifile = codecs.open(csvFile, 'rb', encoding="utf-8-sig")
            reader = csv.reader(ifile)

            lrows = []
            rownum = 0

            for row in reader:
                if rownum == 0:
                    header = row
                    rownum += 1
                else:
                    colnum = 0
                    dRow = DataRow.DataRow()
                    dRow.verbose = self.verbose

                    # Append static values outside of the csv like stock symbol etc
                    for entry in appendCols:
                        for key in entry:
                            dRow.setMember(key, str(entry['idx']), entry[key])
                        # efl
                    # efl

                    if self.verbose:
                        print ('')
                    # eif

                    for col in row:
                        if self.verbose:
                            print (' %-8s: %s' % (header[colnum], col))
                        #eif

                        colName = header[colnum]
                        if len(dataMap) > 0:
                            try:
                                memberIdx = dataMap[colName]
                                if int(memberIdx) != -1:
                                    dRow.setMember(colName, int(memberIdx), col)
                                # eif
                            except:
                                if self.verbose:
                                    print ("Error setting member with index: ", colnum, " with value: ", col)
                                    print ("Unexpected error:", sys.exc_info()[0])
                                # eif
                            # etry

                        else:
                            dRow.setMember(colName, colnum, col)

                        # eif

                        colnum += 1
                    # efl

                    if self.verbose:
                        if self.limitLoad:
                          dRow.printDataRow()
                        # eif
                    # eif

                    dRow.stampId()
                    lrows.append(dRow.copy())
                    rownum += 1
                    self.rowCount += 1
                #eif

                if self.limitLoad == True and self.rowCount >= self.rowLimit and self.rowLimit > 0:
                    break;
                #eif

            #efl

            ifile.close()

            print ("Loaded %i rows from this data file." % (rownum))
            lrows = self.sortRows(lrows)
            self.cleanRows(lrows)
            self.rows.extend(lrows)

            print ('CleanCount: %i RowCount: %i RowsFound: %i' % (self.cleanCount, self.rowCount, len(self.rows)))
        # eif
    # edef

    def resetRows(self):
        self.rows = []
        self.rowCount = 0
        self.cleanCount = 0
    # edef

    def sortRows(self, lrows):
        return sorted(lrows, key=id)
    # edef

    def cleanRows(self, lrows):
        if self.cleanData == True:
            print ("Cleaning row data...")
            should_restart = True
            while should_restart:
                should_restart = False
                for row in lrows:
                    if row.error == True:
                        lrows.remove(row)
                        self.rowCount -= 1
                        self.cleanCount += 1
                        should_restart = True
                    # eif
                # efl
            # fwl
        # eif
    # edef

# eclass